import kraken.entrytypes.MetaData as MetaData
import kraken.tools.TextUi as TextUi
import kraken.entrytypes.system.OwnerData as OwnerData
import kraken.query.GenerateQuerySet as GenerateQuerySet
import os.path

def description(system, runtime):
    return "Shows the configuration of the kraken."
    

def execute(system, runtime):
   commands = [ 
      "Show configuration",
      "Show summary report",
      "Edit configuration",
      "-",
      "Edit owner",
      "-",
      "Create the initial directories of the kraken",
      "Create a standard set of queries",
      "Check the kraken directories",
      "Check the consistency of the kraken"
      "-",
      "Exit"            
   ]
   commandname = TextUi.inputMenu("Select the command to execute:", commands)
   if commandname == "Show configuration":
       print "\n" + str(system)    
       return
   if commandname == "Show summary report":
       metadata = MetaData.loadMetaData(system, runtime)
       print "\n" + str(metadata)
       return
   if commandname == "Edit configuration":   
       system.edit()
       return
   if commandname == "Edit owner":
       editOwner(system)
       return
   if commandname == "Create the initial directories of the kraken":
       createInitialDirectories(system, create = 1)
       return
   if commandname == "Create a standard set of queries":
       createStandardSetOfFilters(system, runtime)
       return
   if commandname == "Check the kraken directories":
       createInitialDirectories(system, create = 0)
       return
   if commandname == "Check the consistency of the kraken":
       checkConsistency(system, runtime)
       return
   

def createStandardSetOfFilters(system, runtime):
    metadata = MetaData.loadMetaData(system, runtime)
    GenerateQuerySet.generateQuerySet(system, metadata)
    
errors = 0
checks = 0
creates = 0
    
def checkAndCreateDir(system, directory, create):
    global errors, checks, creates
    checks += 1
    system.addProperty(directory, os.path.normpath(system.getProperty(directory)))
    dirname = system.getProperty(directory)
    if (not os.path.isdir(dirname)):
       if create:
          try: 
             os.mkdir(dirname)
          except:
             logging.critical("Can not create directory " + dirname)
             errors += 1
       else: 
           logging.critical("The destination directory does not exist:" + 
              system.getProperty("destinationDir"))
           errors += 1
      
    
def createInitialDirectories(system, create):
    global errors, checks, creates
    checks = 0
    errors = 0
    creates = 0
    
    # Convert all the paths to the Unix path
    print "Checking destination directory..."
    checkAndCreateDir(system, "destinationDir", create)
                
    # the kraken dirs            
    print "Checking kraken directories..."
    checks += 1
    newDirs = []
    for a in system.getProperty("krakenDirs"):
        kdir = os.path.normpath(a)
        if (not os.path.isdir(kdir)):
           logging.critical("The kraken source directory does not exist:" + kdir)
           errors += 1
        newDirs.append(kdir)
    system.addProperty("krakenDirs", newDirs)
    
    # destination directory is a kraken dir.
    print "Checking if the destination dir is a kraken dir..."
    checks += 1
    if not (system.getProperty("destinationDir") in system.getProperty("krakenDirs")):
       logging.critical("The destination directory " + system.getProperty("destinationDir") + 
           " is not one of the kraken directories " + str(system.getProperty("krakenDirs")))
       errors += 1

    # the directory of the generated entries
    print "Checking the directory of generated entries..."
    checkAndCreateDir(system, "generatedEntryDir", create)
            
    # the staging dir            
    print "Checking the temporary (staging) directory..."
    checkAndCreateDir(system, "tempDir", create)

    # the system dir            
    print "Checking the system directory..."
    checkAndCreateDir(system, "systemDir", create)
        
    # the filter dir            
    print "Checking the system directory..."
    checkAndCreateDir(system, "filterDir", create)    
    
    # the html            
    print "Checking the html directory..."
    checkAndCreateDir(system, "htmlDir", create)        

    if errors == 0:
        print "\n\nThere were no errors. " + str(checks) + " checks performed.\n"
    else:
        print "\n\n" + str(errors) + " found out of " + str(checks) + " checks performed.\n"

def checkConsistency(system, runtime):
    global errors, checks, creates
    errors = 0
    metadata = MetaData.loadMetaData(system, runtime)
    ids = {}
    for entry in metadata.entries:
        checks += 1
        identifier = entry.getResourceURI()
        if identifier in ids:
            errors += 1
            logging.critical("Duplicate entries for id =" + identifier)
            logging.critical("Entry 1" + entry.getProperty("title"))
            logging.critical("Entry 2" + ids[identifier].getProperty("title"))
        else:
            ids[identifier] = entry
    if errors == 0:
        print "\n\nThere were no duplicate identifiers in the " + str(checks) + " entries.\n"
    else:
        print "\n\n" + str(errors) + " duplicate identifiers found in " + str(checks) + " entries.\n"
    
def editOwner(system):
    """
       Edit the owner of the system
    """
    entry = OwnerData.getOwner(system)
    print entry
    OwnerData.editOwner(entry, system)
    OwnerData.updateOwner(entry, system)
   
