import kraken.tools.TextUi as TextUi
import Filters
import Sorters
import Query 
import Classifiers
import Limiter 
import kraken.Constants
import os.path
import os
import pickle

def generateQuerySet(system, metadata):    
    # generate a query for every entry type
    opinionRanking = TextUi.confirm("Sort the entries by opinion ranking?")
    
    for entryType in metadata.getEntryTypes():
        que = Query.Query()
        que.name = entryType        
        que.queryElements.append(getEntryTypeFilter(entryType))
        
        srtr = Sorters.ReverseSorter()
        srtr.theSorter = Sorters.MultiLevelSorter()
        
        s0 = Sorters.PreferredValueSorter()
        s0.propertyName = "keywords"
        s0.notPreferredValues = ["OBSOLETE"]
        srtr.theSorter.sorters.append(s0)
        
        s1 = Sorters.EvaluationSorter()
        s1.propertyName = "Opinion.ranking"
        srtr.theSorter.sorters.append(s1)
        
        que.queryElements.append(srtr)
        
        queryFile = os.path.join(system.getProperty("filterDir"), 
            entryType + "_generated" + kraken.Constants.EXTENSION_QUERY)
        FILE = open(queryFile, "w")    
        pickle.dump(que, FILE)
        FILE.close()
    # generate a filter for unknown entry type:    
    que = Query.Query()
    que.name = "Unknown entry"        
    que.queryElements.append(getUnknownEntryTypeFilter(metadata.getEntryTypes()))
        
    srtr = Sorters.ReverseSorter()
    srtr.theSorter = Sorters.EvaluationSorter()
    srtr.theSorter.propertyName = "Opinion.ranking"
    que.queryElements.append(srtr)
        
    queryFile = os.path.join(system.getProperty("filterDir"), 
         "UnknownEntry" + "_generated" + kraken.Constants.EXTENSION_QUERY)
    FILE = open(queryFile, "w")    
    pickle.dump(que, FILE)
    FILE.close()
    
        
    # Small tables
    for entryType in ["Conference", "ResearchIdea"]:
        que = Query.Query()
        que.name = entryType        
        que.queryElements.append(getEntryTypeFilter(entryType))
        
        srtr = Sorters.ReverseSorter()
        srtr.theSorter = Sorters.EvaluationSorter()
        srtr.theSorter.propertyName = "Opinion.ranking"
        que.queryElements.append(srtr)

        lim = Limiter.Limiter()
        lim.limit = 20
        que.queryElements.append(lim)
        
        queryFile = os.path.join(system.getProperty("filterDir"), 
            entryType + "_smalltable" + kraken.Constants.EXTENSION_QUERY)
        FILE = open(queryFile, "w")    
        pickle.dump(que, FILE)
        FILE.close()
        
        
        
def getEntryTypeFilter(entryType):
        flt = Filters.OrFilter()
        
        flt2 = Filters.ContainsAllFilter()
        flt2.propertyName = "entryTypes"
        flt2.filterValue = [entryType]
        flt.filters.append(flt2)
        
        # this is needed for legacy reasons
        flt2 = Filters.ContainsAllFilter()
        flt2.propertyName = "keywords"
        flt2.filterValue = [entryType]
        flt.filters.append(flt2)
 
        return flt
        
def getUnknownEntryTypeFilter(entryTypes):
        flt = Filters.NotFilter()
        
        flt2 = Filters.ContainsAnyFilter()
        flt2.propertyName = "entryTypes"
        flt2.filterValue = entryTypes
        flt.flt = flt2        
 
        return flt        
