import os
import string
import shutil
import copy
import logging
import datetime
import time
import sys
import kraken.Constants
import strptime

# this is fixing the problem that in windows we don't have readline
if os.name == 'posix': 
    import readline
    haveReadline = 1
else:   
    haveReadline = 0


'''
    Helper function. A relatively sophisticated stuff 
    to allow initilizing the readline stuff with some data
'''        
def inputWithDefault(varname, initvalue=""):
        # this fixes some frequent occurences of editing uninitialized 
        # variables
        if initvalue == None:
           initvalue = ""
        if haveReadline:          
            def __defaultStartupHook():
                readline.insert_text(__currentDefault)        
            __currentDefault = initvalue        
            readline.set_startup_hook(__defaultStartupHook)
            temp = raw_input(varname + " = ")
            readline.set_startup_hook()
        else:
            print varname + " = " + initvalue + "\n\t(<ENTER> to keep)"
            temp = raw_input(varname + " = ")
            if temp == "":
                return initvalue
        return temp
        
def inputString(prompt, initvalue = ""):    
    return inputWithDefault(prompt, initvalue)
    
         
'''
    Helper function. A relatively sophisticated stuff 
    to allow initilizing the readline stuff with some data
'''        
def inputInteger(prompt, initvalue = 0):
     initstring = str(initvalue)
     valuestring = inputWithDefault(prompt, initstring)
     return int(valuestring)

'''
    Helper function. A relatively sophisticated stuff 
    to allow initilizing the readline stuff with some data
'''
def inputSequence(varname, initvalue = [], nsstringlist = []):
    stringlist = copy.copy(nsstringlist)
    stringlist.sort()
    while 1:
        done = 1
        tmp = ""
        for a in initvalue:
            tmp = tmp + a + ","
        tmp = tmp[:-1]
        tmp = inputWithDefault(varname, tmp)
        if tmp != "":
           temp = string.split(tmp, ",")
           temp = map(string.strip, temp)
        else:
           temp = []
        if stringlist != []:
          for a in temp:
             if a not in stringlist:
                 print a + " is not a recognized value here."
                 print "The list of suggested choices are: " + str(stringlist)
                 if not confirm("Do you want to proceed (Y) or continue editing (N) ?"):
                     done = 0
                     break; 
        if done: return temp

def inputDate(prompt, initvalue):        
     while(1):
        try:     
           temp = inputString(prompt, time.strftime(kraken.Constants.FORMAT_TIME, initvalue))
           val2 = strptime.strptime(temp, kraken.Constants.FORMAT_TIME)           
           return val2
        except:
           print "Bad date format. Please use:" + kraken.Constants.FORMAT_TIME
        
'''
   Returns 0 or 1 depending of the confirmation of the question
'''
def confirm(question):
      reply = raw_input(question + " (Y/N) ")
      if reply not in "yY":
          return 0
      return 1            

'''
   Input menu
'''
def inputMenu(prompt, labels, values=None, question = "Enter your selection: ",
   forceSelection = 1, default = None):
    print prompt
    separator = "-" * 60
    if values == None: 
        values = labels
    nums = {}
    while 1:        
      i = 1
      print "\n\n" + separator    
      for a in labels:
         if a == "-":
            print separator         
            continue
         if a == default:
            print "[" + str(i) + "]" + "\t" + a
         else:
            print str(i) + "\t" + a
         nums[i]=a
         i = i + 1
      print separator    
      print ""
      reply = raw_input(question)
      if ((not forceSelection) or (default != None))  and reply == "":
          return default
      # try to interpret it as a number
      selectedLabel = None
      try:
        repval = int(reply)
        selectedLabel = nums[repval] 
      except:
        pass
      if selectedLabel == None:
          matches = []
          for a in labels:
              if string.find(string.lower(a), string.lower(reply)) != -1:
                  matches.append(a)
          if len(matches) > 1:
              print "There are more than one matches:  " + string.join(matches,", ") 
              continue
          if len(matches) == 0:
              print "No menu entry matches this value!"
              continue
          selectedLabel = matches[0]                    
      # ok, we got the label, return the value
      for i in xrange(len(labels)):
          if labels[i] == selectedLabel:
              return values[i]
      

'''      
   Pick files from a directory
'''
def selectFilesFromDirectory(directory, alreadySelected = None, ignore = []):
    DONE = ".....DONE....."
    RESTART = ".....RESTART....."

    def createMenu(directory, ignore):
       dirfiles = os.listdir(directory)
       dirfiles.append("-")
       dirfiles.append(DONE)
       dirfiles.append(RESTART)
       for a in ignore:
           if a in dirfiles:
              dirfiles.remove(a)
           else:
              print a + " is not there"
       return dirfiles
           
    dirfiles = createMenu(directory, ignore)        
    selectedFiles = []

    if alreadySelected != None:
       selectedFiles = alreadySelected
       for a in selectedFiles:
           try:
              dirfiles.remove(a)
           except:
              print "Cannot remove " + a + " from " + str(dirfiles)
    while 1:
        question = "Current selection is: " + str(selectedFiles) + "\n\nPick file to select: "
        v = inputMenu("Select from directory", dirfiles, question=question, forceSelection = 0)
        if v == DONE or v==None:
            return selectedFiles
        elif v == RESTART:
            dirfiles = createMenu(directory, ignore)
            #dirfiles = os.listdir(directory)
            #dirfiles.append(DONE)
            #dirfiles.append(RESTART)
            selectedFiles = []
        else:
            selectedFiles.append(v)
        dirfiles.remove(v)
        print "Currently selected: " + str(selectedFiles)

'''      
   Select a word from a list of words
'''
def selectWordsFromList(wordlist, alreadySelected = None):
    DONE = ".....DONE....."
    RESTART = ".....RESTART....."

    def createMenu(wordlist):
       menuitems = copy.copy(wordlist)
       menuitems.append("-")
       menuitems.append(DONE)
       menuitems.append(RESTART)
       return menuitems
           
    menuitems = createMenu(wordlist)        
    selectedWords = []

    if alreadySelected != None:
       selectedWords = alreadySelected
       for a in selectedWords:
           try:
              menuitems.remove(a)
           except:
              print "Cannot remove " + a + " from " + str(menuitems)
    while 1:
        question = "Current selection is: " + str(selectedWords) + "\n\nPick word to select: "
        v = inputMenu("Select from list:", menuitems, question=question, forceSelection = 0)
        if v == DONE or v==None:
            return selectedWords
        elif v == RESTART:
            menuitems = createMenu(wordlist)
            selectedWords = []
        else:
            selectedWords.append(v)
        menuitems.remove(v)
        print "Currently selected: " + str(selectedWords)
        
        
"""
   Returns a sequence of iteration.
"""
def inputEntryTypes(metadata, default = []):
    allowed_types = metadata.getEntryTypes()
    # print "Allowed values: " + str(allowed_types)
    # value = inputSequence("Entry types",
    #    initvalue = default, nsstringlist=allowed_types)
    # print "Entry types list:" + str(value)    
    return selectWordsFromList(allowed_types, default)                   
   
longCommandCounter = None   
   
def startLongCommand(text = "Starting..."):
    global longCommandCounter
    longCommandCounter = datetime.datetime.now()
    sys.stdout.write(text)
    sys.stdout.flush()
 
def endLongCommand(text = "Done.", newline = 0):
    global longCommandCounter
    d = datetime.datetime.now() - longCommandCounter
    if newline == 1:
        sys.stdout.write("\n")
    sys.stdout.write(text + " (" + str(d.seconds + int(d.microseconds/1000)/1000.0) + " secs).\n")
    sys.stdout.flush()    
    return d 
   
'''
  Input menu
'''
def test():
        #print inputMenu("Select from the list", ("Here", "We", "Are"))
        #print selectFilesFromDirectory(".", ["MANIFEST.in"]) 
        print inputDate("Enter the time", time.localtime())
      