"""
Python-PDI - Library for Personal Data Interchange.
Copyright (C) 2002-2003 Peter Gebauer

This is the vCalendar base library.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
version 2 as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""

from pdi.core import *

class VCalendar(Component):
    """The base of ICalendar component. Use ICalendar instead if you want RFC2445 compliancy!"""

    def __init__(self, parent = None):
        """
        @type   parent: L{pdi.core.Component}
        @param  parent: The parent component, if any. May be omitted or None if it is a top-level component.
        """
        super(VCalendar, self).__init__(parent)
        self.registerComponents([VEvent, VTodo, VJournal], RULE_MAY)
        self.registerComponents([VCalendar], RULE_NOT)
        self.registerProperties(['PRODID', 'VERSION'], RULE_MUST)

class VEvent(Component):
    """Event component, sub-component to VCalendar."""

    def __init__(self, parent = None):
        """
        @type   parent: L{pdi.core.Component}
        @param  parent: The parent component, if any. May be omitted or None if it is a top-level component.
        """
        super(VEvent, self).__init__(parent)
        self.registerProperties(['UID', 'SUMMARY'], RULE_MUST)
        self.registerComponents([VEvent], RULE_NOT)
        self.registerProperties(['DTSTAMP',
                                 'DTSTART',
                                 'RRULE',
                                 'DTEND'
                                 ], RULE_MAY)

class VTodo(Component):
    """Todo component, sub-component to VCalendar."""

    def __init__(self, parent = None):
        """
        @type   parent: L{pdi.core.Component}
        @param  parent: The parent component, if any. May be omitted or None if it is a top-level component.
        """
        super(VTodo, self).__init__(parent)
        self.registerComponents([VTodo], RULE_NOT)

class VJournal(Component):
    """Journal component, sub-component to VCalendar. This is not a stand-alone component!"""

    def __init__(self, parent = None):
        """
        @type   parent: L{pdi.core.Component}
        @param  parent: The parent component, if any. May be omitted or None if it is a top-level component.
        """
        super(VJournal, self).__init__(parent)
        self.registerComponents([VJournal], RULE_NOT)
